# Copyright (C) 2013, Eric Wong <normalperson@yhbt.net> and all contributors
# License: GPLv3 or later (https://www.gnu.org/licenses/gpl-3.0.txt)
require_relative 'server_helper'

class TestExtrasExecCGI < Testcase
  ENV["N"].to_i > 1 and parallelize_me!
  include ServerHelper
  alias setup server_helper_setup
  alias teardown server_helper_teardown

  def test_exec_cgi
    err, cfg, host, port = @err, Yahns::Config.new, @srv.addr[3], @srv.addr[1]
    runme = "#{Dir.pwd}/test/test_extras_exec_cgi.sh"
    assert File.executable?(runme), "run test in project root"
    pid = mkserver(cfg) do
      require './extras/exec_cgi'
      cfg.instance_eval do
        app(:rack, ExecCgi.new(runme)) do
          listen "#{host}:#{port}"
        end
        stderr_path err.path
      end
    end

    Timeout.timeout(30) do # we can chunk
      c = get_tcp_client(host, port)
      c.write "GET / HTTP/1.1\r\nConnection: close\r\n" \
              "Host: example.com\r\n\r\n"
      head, body = c.read.split(/\r\n\r\n/, 2)
      assert_match %r{^Transfer-Encoding: chunked\b}, head
      assert_equal "5\r\nHIHI\n\r\n0\r\n\r\n", body
      c.close
      cerr = tmpfile(%w(curl .err))
      assert_equal "HIHI\n", `curl -sSfv 2>#{cerr.path} http://#{host}:#{port}/`
      assert_match %r{\bTransfer-Encoding: chunked\b}, cerr.read
      cerr.close!
    end

    Timeout.timeout(30) do # do not chunk on clients who can't handle chunking
      c = get_tcp_client(host, port)
      c.write "GET / HTTP/1.0\r\nHost: example.com\r\n\r\n"
      head, body = c.read.split(/\r\n\r\n/)
      assert_equal "HIHI\n", body
      refute_match %r{^Transfer-Encoding: chunked\b}, head
      c.close
    end

    Timeout.timeout(30) do # sure env is sane
      c = get_tcp_client(host, port)
      c.write "GET /env\r\n\r\n"
      head, body = c.read.split(/\r\n\r\n/)
      assert_nil body
      assert_match %r{^REQUEST_METHOD=GET$}, head
      assert_match %r{^PATH_INFO=/env$}, head
      assert_match %r{^QUERY_STRING=$}, head
      c.close
    end

    Timeout.timeout(30) do # known length should not chunk
      c = get_tcp_client(host, port)
      c.write "GET /known-length HTTP/1.1\r\nConnection: close\r\n" \
              "Host: example.com\r\n\r\n"
      head, body = c.read.split(/\r\n\r\n/, 2)
      refute_match %r{^Transfer-Encoding: chunked\b}, head
      assert_match %r{^Content-Length: 5\b}, head
      assert_equal "HIHI\n", body
      c.close
    end

    Timeout.timeout(30) do # 404
      c = get_tcp_client(host, port)
      c.write "GET /not-found HTTP/1.0\r\n\r\n"
      head, body = c.read.split(/\r\n\r\n/)
      assert_match %r{\AHTTP/1\.1 404 Not Found}, head
      assert_nil body
      c.close
    end
  ensure
    quit_wait(pid)
  end
end
